@@ -167,7 +167,7 @@ count_lesions <- function(img,
       if(is.character(img)){
         all_files <- sapply(list.files(diretorio_original), file_name)
         check_names_dir(img, all_files, diretorio_original)
-        imag <- list.files(diretorio_original, pattern = img)
+        imag <- list.files(diretorio_original, pattern = paste0("^",img, "\\."))
         name_ori <- file_name(imag)
         extens_ori <- file_extension(imag)
         img <- image_import(paste(diretorio_original, "/", name_ori, ".", extens_ori, sep = ""))
@@ -452,7 +452,6 @@ count_lesions <- function(img,
       marker <- ifelse(is.null(marker), "point", marker)
       marker_col <- ifelse(is.null(marker_col), "white", marker_col)
       marker_size <- ifelse(is.null(marker_size), 0.9, marker_size)
-
       if(show_image == TRUE){
         if(marker == "text"){
           image_show(im2)
@@ -523,8 +522,9 @@ count_lesions <- function(img,
                   statistics = c("n", "min", "mean", "max", "sd", "sum", "prop"))
       stats <- stats[c(3, 1, 2)]
       shape <- shape[,c(1:6, 8:9, 7)]
+      shape <- transform(shape, radius_ratio = s.radius.max / s.radius.min)
       colnames(shape) <- c("id", "x", "y", "area", "perimeter", "radius_mean",
-                           "radius_min", "radius_max", "radius_sd")
+                           "radius_min", "radius_max", "radius_sd", "radius_ratio")
       results <- list(results = shape,
                       statistics = stats)
       class(results) <- "plm_count"
@@ -581,8 +581,10 @@ count_lesions <- function(img,
       results <- list()
       pb <- progress(max = length(plants), style = 4)
       for (i in 1:length(plants)) {
+        if(verbose == TRUE){
         run_progress(pb, actual = i,
                      text = paste("Processing image", names_plant[i]))
+        }
         results[[i]] <-
           help_count(img  = names_plant[i],
                      img_healthy, img_lesion, img_background, randomize,
